//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "algo.h"
#include "repere.h"
#include "box.h"
#define ny 15
//---------------------------------------------------------------------------
#pragma package(smart_init)
interval UX1[ny+1],UX2[ny+1],UY[ny+1];   //Contain all feasible x1(k),x2(k) and y(k)
interval X1[ny+1],X2[ny+1],Y[ny+1],Ym[ny+1];
//--------------------------------------------------------------------
void Init()
{   double y[ny+1],x1[ny+1],x2[ny+1];
    x1[0]=-1; x2[0]=0;
    for(int k=1;k<=ny;k++)       // Generation of the interval data
    {     x1[k]=0.1*x1[k-1]+x2[k-1]*exp(x1[k-1]);
          x2[k]=x1[k-1]+0.1*x2[k-1]*x2[k-1]+sin(k);
          y[k]=(x2[k]/x1[k])+0.5*sin(100*k); //sin(100k) -> for the noise
          Y[k]=y[k]+0.5*interval(-1,1);
    }
    for(int k=0;k<=ny;k++)       // At the moment, no feasible values for x1(k),x2(k),y(k)
    {     UX1[k].isEmpty=true;   // have been found
          UX2[k].isEmpty=true;
          UY[k].isEmpty=true;
    }
}
//--------------------------------------------------------------------
iboolean Contract(repere& R,box& P)
{   interval Z1[ny+1],Z2[ny+1],Z3[ny+1],Z4[ny+1];
    iboolean dedans=itrue;
    if (P.isEmpty) return (ifalse);
    box Q(2,true);
    do
    {   Q=P;
        R.DrawCadre (P,clBlack,psSolid,1,2);   //To see the contractions
        X1[0]=P[1];
        X2[0]=P[2];
        for(int k=1;k<=ny;k++)       //Forward propagation
        {   Z1[k]=Exp(X1[k-1]);
            Z2[k]=X2[k-1]*Z1[k];
            X1[k]=0.1*X1[k-1]+Z2[k];
            Z3[k]=0.1*Sqr(X2[k-1]);
            Z4[k]=Z3[k]+sin(k);
            X2[k]=X1[k-1]+Z4[k];
            Ym[k]=X2[k]/X1[k];
            Ym[k]=Inter(Ym[k],Y[k]);
            if (Ym[k].isEmpty) {P.isEmpty=true;return ifalse;}
        }
        for(int k=ny;k>=1;k--)       //Backward propagation
        {   X2[k]     =Inter(Ym[k]*X1[k],X2[k]);
            X1[k]     =Inter(X2[k]/Ym[k],X1[k]);
            X1[k-1]   =Inter(X2[k]-Z4[k],X1[k-1]);
            Z4[k]     =Inter(X2[k]-X1[k-1],Z4[k]);
            Z3[k]     =Inter(Z4[k]-sin(k),Z3[k]);
            X2[k-1]   =Inter(Sqrt(10*Z3[k]),X2[k-1]);
            X1[k-1]   =Inter(10*(X1[k]-Z2[k]),X1[k-1]);
            Z2[k]     =Inter(X1[k]-0.1*X1[k-1],Z2[k]);
            X2[k-1]   =Inter(Z2[k]/Z1[k],X2[k-1]);
            Z1[k]     =Inter(Z2[k]/X2[k-1],Z1[k]);
            X1[k-1]   =Inter(Log(Z1[k]),X1[k-1]);
        }
        P[1]=X1[0];
        P[2]=X2[0];
        if ((P[1].isEmpty)||(P[2].isEmpty))  {P.isEmpty=true; return ifalse;}
    }
    while (Eloignement2(Q,P)>0.00001);
    return iperhaps;
};
//--------------------------------------------------------------------
void SIVIA(repere& R,box& P)
{   box P1(2,true);
    box P2(2,true);
    if (Width(P)<0.01)     //if the current box is too small STOP
    {     R.DrawBox(P,clYellow,clYellow,bsSolid);
          UX1[0]=Union(UX1[0],X1[0]);
          UX2[0]=Union(UX2[0],X2[0]);
          for (int k=1;k<=ny;k++)
          {  UY[k] =Union(Ym[k],UY[k]);
             UX1[k]=Union(UX1[k],X1[k]);
             UX2[k]=Union(UX2[k],X2[k]);
          }
          return;
    };
    iboolean test=Contract(R,P);
    if (test==itrue)   {  R.DrawBox(P,clRed,clPurple,bsSolid);    };
    if (test!=ifalse)  {  Bisect(P,P1,P2);  SIVIA(R,P1);  SIVIA(R,P2);    }
    else R.DrawBox(P,clBlue,clAqua,bsSolid);
} ;
//--------------------------------------------------------------------
void Go(repere& R,box& P,int i)
{   if (i==0)  //First click on "GO"
    //Characterize the set of feasible initial states (x1(0),x2(0))
    {       Init(); SIVIA(R,P);}

    if (i==1)  //Second click on "GO"
    //Draws the contracted intervals for x1(k) at times 0,1,...,10
    {       R.interval_x=interval(-1,ny+1);
            R.interval_y=interval(-2,2);
            box X10(2,true); X10[1]=R.interval_x; X10[2]=R.interval_y;
            R.DrawBox(X10,clWhite,clWhite,bsSolid);
            box BX1(ny+1,true);
            box T(ny+1,true);
            for (int i=1;i<=ny+1;i++) {BX1[i]=UX1[i-1];T[i]=interval(i-1,i-1);};
            R.DrawData(BX1,T,clBlack,0.01);
    }

    if (i==2)  //Third click on "GO"
    //Draws the contracted intervals for x2(k) at times 0,1,...,10
    {       R.interval_x=interval(-1,ny+1);
            R.interval_y=interval(-2,2);
            box X20(2,true); X20[1]=R.interval_x; X20[2]=R.interval_y;
            R.DrawBox(X20,clWhite,clWhite,bsSolid);
            box BX2(ny+1,true),T(ny+1,true);
            for (int i=1;i<=ny+1;i++) {BX2[i]=UX2[i-1];T[i]=interval(i-1,i-1);};
            R.DrawData(BX2,T,clBlack,0.01);
    }

    if (i==3)  //4th click on "GO"
    //Draws the contracted (or filtered) intervals for the output y(k) at times 1,...,10
    {       R.interval_x=interval(-1,ny+1);
            R.interval_y=interval(-15,15);
            box Y0(2,true); Y0[1]=R.interval_x; Y0[2]=R.interval_y;
            R.DrawBox(Y0,clWhite,clWhite,bsSolid);
            box BY(ny,true),T(ny,true);
            for (int i=1;i<=ny;i++) {BY[i]=UY[i];T[i]=interval(i,i);};
            R.DrawData(BY,T,clBlack,0.02);
    }

    if (i==4)  //5th click on "GO"
    //Draws the initial intervals for the output y(k) at times 1,...,10
    {       R.interval_x=interval(-1,ny+1);
            R.interval_y=interval(-15,15);
            box Y0(2,true); Y0[1]=R.interval_x; Y0[2]=R.interval_y;
            R.DrawBox(Y0,clWhite,clWhite,bsSolid);
            box BY(ny,true),T(ny,true);
            for (int i=1;i<=ny;i++) {BY[i]=Y[i];T[i]=interval(i,i);};
            R.DrawData(BY,T,clBlack,0.02);
    }
} ;

